var IrvineService = {
  _UConv: null,
  MAX_ARG_LENGTH: 1000,

  get UConv() {
    if (!this._UConv) {
      this._UConv = Components
                      .classes["@mozilla.org/intl/scriptableunicodeconverter"]
                      .getService(Components.interfaces
                                            .nsIScriptableUnicodeConverter);
    }

    return this._UConv;
  },

  createObject: function(aMethodName, aParams, aBlock) {
    var filename = nsPreferences.copyUnicharPref("extensions.irvine.edtPath",
                                                 "");
    var file = Components.classes["@mozilla.org/file/local;1"]
                         .createInstance(Components.interfaces.nsILocalFile);
    var process = Components.classes["@mozilla.org/process/util;1"]
                            .createInstance(Components.interfaces.nsIProcess);

    file.initWithPath(filename);
    process.init(file);

    var args = ["-object", "Irvine.Api", "-method", aMethodName];

    for (var i = 0; i < aParams.length; i++) {
      args.push(aParams[i]);
    }

    process.run(aBlock, args, args.length, {});
  },

  get onLink() {
    return (gContextMenu.onSaveableLink || gContextMenu.onImage);
  },

  get linkURI() {
    return (gContextMenu.onSaveableLink) ?
            (typeof(gContextMenu.linkURL) == "function" ?
            gContextMenu.linkURL() : gContextMenu.linkURL) :
            (gContextMenu.onImage) ? gContextMenu.imageURL : null;
  },

  get linkURIs() {
    var a = document.popupNode.ownerDocument.links;
    var links = [];

    for (var i = 0; i < a.length; i++) {
      links.push(a[i].href);
    }

    return links;
  },

  get imageURIs() {
    var img = document.popupNode.ownerDocument.images;
    var uris = [];

    for (var i = 0; i < img.length; i++) {
      uris.push(img[i].src);
    }

    return uris;
  },

  get linkText() {
    if (gContextMenu.onSaveableLink) {
      this.UConv.charset = "Shift_JIS";
      var text = gContextMenu.linkText();
      text = this.UConv.ConvertFromUnicode(text);

      return text;
    } else {
      return "";
    }
  },

  get selectionText() {
    return document.popupNode.ownerDocument.defaultView
                   .getSelection().toString();
  },

  get locationURI() {
    return document.popupNode.ownerDocument.location.href;
  },

  getSelectLinkCollection: function() {
    var selection = document.commandDispatcher.focusedWindow.getSelection();
    var links = document.commandDispatcher.focusedWindow.document.links;
    var urls = [];

    if(!links) {
      return;
    }

    for (var i = 0; i < links.length; i++) {
      if (selection.containsNode(links[i], true)) {
        urls.push(links[i].href);
      }
    }

    return urls;
  },

  removeURLHash: function(aURL) {
    return aURL.replace(/#[^#]+$/, "");
  },

  downloadLink: function() {
    var url = this.linkURI;

    if (url) {
      this.createObject("Download", [url, 0], false);
    }
  },

  downloadLinkNow: function() {
    var url = this.linkURI;

    if (url) {
      this.createObject("Download", [url, 3], false);
    }
  },

  addLink: function() {
    var text = this.selectionText;
    var links = [];
    var irvineFlag = 0;

    if (text) {
      var selectLinks = this.getSelectLinkCollection();
      if (selectLinks.length > 0) {
        links = selectLinks;
      } else {
        links = text.split("\n");
      }
    } else if (this.onLink) {
      links.push(this.linkURI);
    } else {
      links = links.concat(this.linkURIs);
      irvineFlag = 1;
    }

    var urls = "";
    var argLength = 0;

    for (var i in links) {
      argLength += links[i].length;

      if (argLength > this.MAX_ARG_LENGTH) {
        this.createObject("AddUrl", [urls, irvineFlag], true);
        urls = this.removeURLHash(links[i]) + "\n";
        argLength = links[i].length;
      } else {
        urls = urls + this.removeURLHash(links[i]) + "\n";
      }
    }

    if (urls) {
      this.createObject("AddUrl", [urls, irvineFlag], false);
    }
  },

  downloadLinkAs: function() {
    var url = this.linkURI;

    if (url) {
      this.createObject("Download", [url, 1], false);
    }
  },

  downloadLinkWithComment: function() {
    var url = this.linkURI;

    if (url) {
      url = "Url=" + url + "\n" + "Comment=" + this.linkText + "\n";
      this.createObject("AddIRI", [url], false);
    }
  },

  downloadLinkWithSetting: function() {
    var url = this.linkURI;

    if (url) {
      url = "Url=" + url + "\n" +
            "Referer=" + this.locationURI + "\n" +
            "Comment=" + this.linkText + "\n";

      this.createObject("CreateQueueItem", [url, false], false);
    }
  },

  downloadLinkWithCueFolderSelect: function() {
    var url = this.linkURI;

    if (url) {
      this.createObject("Download", [url, 2], false);
    }
  },

  downloadAllLinks: function() {
    var links = this.linkURIs;
    var urls = "";
    var argLength = 0;

    for (var i in links) {
      argLength += links[i].length;

      if (argLength > this.MAX_ARG_LENGTH) {
        this.createObject("AddUrlAndReferer", [urls, this.locationURI, 1],
                          true);
        urls = links[i] + "\n";
        argLength = links[i].length;
      } else {
        urls = urls + links[i] + "\n";
      }
    }

    if (urls) {
      this.createObject("AddUrlAndReferer", [urls, this.locationURI, 1],
                        false);
    }
  },

  downloadAllLinksAndImages: function() {
    var links = this.linkURIs;
    var images = this.imageURIs;
    var urls = "";
    var argLength = 0;

    for (var i in links) {
      argLength += links[i].length;

      if (argLength > this.MAX_ARG_LENGTH) {
        this.createObject("AddUrlAndReferer", [urls, this.locationURI, 1],
                          true);
        urls = links[i] + "\n";
        argLength = links[i].length;
      } else {
        urls = urls + links[i] + "\n";
      }
    }

    for (var i in images) {
      argLength += images[i].length;

      if (argLength > this.MAX_ARG_LENGTH) {
        this.createObject("AddUrlAndReferer", [urls, this.locationURI, 1],
                          true);
        urls = images[i] + "\n";
        argLength = images[i].length;
      } else {
        urls = urls + images[i] + "\n";
      }
    }

    if (urls) {
      this.createObject("AddUrlAndReferer", [urls, this.locationURI, 1],
                        false);
    }
  },

  downloadSelection: function() {
    var text = this.selectionText;

    if (text) {
      if (text.match(/\r|\n/)) {
        this.createObject("AddUrl", [text, 1], false);
      } else {
        this.createObject("Download", [text, 0], false);
      }
    }
  },

  importLinks: function() {
    if (this.onLink) {
      var url = this.linkURI;

      if (url) {
        this.createObject("ImportLinks", [url, 0], false);
      }
    } else {
      var links = this.linkURIs;
      var urls = "";
      var argLength = 0;

      for (var i in links) {
        argLength += links[i].length;

        if (argLength > this.MAX_ARG_LENGTH) {
          this.createObject("ImportLinks", [urls, 0], true);
          urls = links[i] + "\n";
          argLength = links[i].length;
        } else {
          urls = urls + links[i] + "\n";
        }
      }

      if (urls) {
        this.createObject("ImportLinks", [urls, 0], false);
      }
    }
  },

  downloadThisPage: function() {
    var url = this.locationURI;

    if (url) {
      this.createObject("AddUrl", [url, 0], false);
    }
  },

  showItems: function() {
    var enabled = nsPreferences.copyUnicharPref("extensions.irvine.edtPath",
                                                "") != "";
    var flat = nsPreferences.getBoolPref("extensions.irvine.flat", false);
    var show = enabled && this.onLink;

    // Cascade type context menu
    gContextMenu.showItem("irvine-downloadLink", show && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.download", false));
    gContextMenu.showItem("irvine-downloadLinkNow", show && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.immediate", false));
    gContextMenu.showItem("irvine-addLink", enabled && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.add", false));
    gContextMenu.showItem("irvine-downloadLinkAs", show && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.saveas", false));
    gContextMenu.showItem("irvine-downloadLinkWithCueFolderSelect",
      show && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.withqueue", false));
    gContextMenu.showItem("irvine-downloadLinkWithComment", show && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.withcomment", false));
    gContextMenu.showItem("irvine-downloadLinkWithSetting", show && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.withsetting", false));
    gContextMenu.showItem("irvine-downloadSelection", enabled &&
      gContextMenu.isTextSelected && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.selection", false));
    gContextMenu.showItem("irvine-downloadAllLinks", enabled && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.all", false));
    gContextMenu.showItem("irvine-downloadAllLinksAndImages",
      enabled && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.allwithimg", false));
    gContextMenu.showItem("irvine-importLinks", enabled && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.import", false));
    gContextMenu.showItem("irvine-downloadThisPage", enabled && !flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.thispage", false));
    gContextMenu.showItem("irvine-separator",
      !document.getElementById("irvine-downloadLink").hidden ||
      !document.getElementById("irvine-downloadLinkNow").hidden ||
      !document.getElementById("irvine-addLink").hidden ||
      !document.getElementById("irvine-downloadLinkAs").hidden ||
      !document.getElementById("irvine-downloadLinkWithCueFolderSelect").hidden ||
      !document.getElementById("irvine-downloadLinkWithComment").hidden ||
      !document.getElementById("irvine-downloadLinkWithSetting").hidden ||
      !document.getElementById("irvine-downloadSelection").hidden ||
      !document.getElementById("irvine-downloadAllLinks").hidden ||
      !document.getElementById("irvine-downloadAllLinksAndImages").hidden ||
      !document.getElementById("irvine-importLinks").hidden ||
      !document.getElementById("irvine-downloadThisPage").hidden);
    gContextMenu.showItem("irvine-menu", !flat &&
      !document.getElementById("irvine-downloadLink").hidden ||
      !document.getElementById("irvine-downloadLinkNow").hidden ||
      !document.getElementById("irvine-addLink").hidden ||
      !document.getElementById("irvine-downloadLinkAs").hidden ||
      !document.getElementById("irvine-downloadLinkWithCueFolderSelect").hidden ||
      !document.getElementById("irvine-downloadLinkWithComment").hidden ||
      !document.getElementById("irvine-downloadLinkWithSetting").hidden ||
      !document.getElementById("irvine-downloadSelection").hidden ||
      !document.getElementById("irvine-downloadAllLinks").hidden ||
      !document.getElementById("irvine-downloadAllLinksAndImages").hidden ||
      !document.getElementById("irvine-importLinks").hidden ||
      !document.getElementById("irvine-downloadThisPage").hidden);

    // Flat type context menu
    gContextMenu.showItem("irvine-downloadLink-flat", show && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.download", false));
    gContextMenu.showItem("irvine-downloadLinkNow-flat", show && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.immediate", false));
    gContextMenu.showItem("irvine-addLink-flat", enabled && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.add", false));
    gContextMenu.showItem("irvine-downloadLinkAs-flat", show && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.saveas", false));
    gContextMenu.showItem("irvine-downloadLinkWithCueFolderSelect-flat",
      show && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.withqueue", false));
    gContextMenu.showItem("irvine-downloadLinkWithComment-flat",
      show && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.withcomment", false));
    gContextMenu.showItem("irvine-downloadLinkWithSetting-flat",
      show && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.withsetting", false));
    gContextMenu.showItem("irvine-downloadSelection-flat", enabled &&
      gContextMenu.isTextSelected && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.selection", false));
    gContextMenu.showItem("irvine-downloadAllLinks-flat", enabled && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.all", false));
    gContextMenu.showItem("irvine-downloadAllLinksAndImages-flat",
      enabled && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.allwithimg", false));
    gContextMenu.showItem("irvine-importLinks-flat", enabled && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.import", false));
    gContextMenu.showItem("irvine-downloadThisPage-flat", enabled && flat &&
      nsPreferences.getBoolPref("extensions.irvine.menu.thispage", false));
    gContextMenu.showItem("irvine-separator-flat", flat &&
      !document.getElementById("irvine-downloadLink-flat").hidden ||
      !document.getElementById("irvine-downloadLinkNow-flat").hidden ||
      !document.getElementById("irvine-addLink-flat").hidden ||
      !document.getElementById("irvine-downloadLinkAs-flat").hidden ||
      !document.getElementById("irvine-downloadLinkWithCueFolderSelect-flat").hidden ||
      !document.getElementById("irvine-downloadLinkWithComment-flat").hidden ||
      !document.getElementById("irvine-downloadLinkWithSetting-flat").hidden ||
      !document.getElementById("irvine-downloadSelection-flat").hidden ||
      !document.getElementById("irvine-downloadAllLinks-flat").hidden ||
      !document.getElementById("irvine-downloadAllLinksAndImages-flat").hidden ||
      !document.getElementById("irvine-importLinks-flat").hidden ||
      !document.getElementById("irvine-downloadThisPage-flat").hidden);

    // Set accesskeys
    if (flat) {
      document.getElementById("irvine-downloadLink-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.download", ""));
      document.getElementById("irvine-downloadLinkNow-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.immediate", ""));
      document.getElementById("irvine-addLink-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.add", ""));
      document.getElementById("irvine-downloadLinkAs-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.saveas", ""));
      document.getElementById("irvine-downloadLinkWithCueFolderSelect-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.withqueue", ""));
      document.getElementById("irvine-downloadSelection-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.selection", ""));
      document.getElementById("irvine-downloadLinkWithComment-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.withcomment", ""));
      document.getElementById("irvine-downloadLinkWithSetting-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.withsetting", ""));
      document.getElementById("irvine-downloadAllLinks-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.all", ""));
      document.getElementById("irvine-downloadAllLinksAndImages-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.allwithimg", ""));
      document.getElementById("irvine-importLinks-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.import", ""));
      document.getElementById("irvine-downloadThisPage-flat")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.thispage", ""));
    } else {
      document.getElementById("irvine-menu")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.irvine", ""));
      document.getElementById("irvine-downloadLink")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.download", ""));
      document.getElementById("irvine-downloadLinkNow")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.immediate", ""));
      document.getElementById("irvine-addLink")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.add", ""));
      document.getElementById("irvine-downloadLinkAs")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.saveas", ""));
      document.getElementById("irvine-downloadLinkWithCueFolderSelect")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.withqueue", ""));
      document.getElementById("irvine-downloadSelection")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.selection", ""));
      document.getElementById("irvine-downloadLinkWithComment")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.withcomment", ""));
      document.getElementById("irvine-downloadLinkWithSetting")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.withsetting", ""));
      document.getElementById("irvine-downloadAllLinks")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.all", ""));
      document.getElementById("irvine-downloadAllLinksAndImages")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.allwithimg", ""));
      document.getElementById("irvine-importLinks")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.import", ""));
      document.getElementById("irvine-downloadThisPage")
              .setAttribute("accesskey", nsPreferences.copyUnicharPref(
                "extensions.irvine.accesskey.thispage", ""));
    }
  },

  initItems: function() {
    var menu = document.getElementById("contentAreaContextMenu");
    var pref = Components.classes["@mozilla.org/preferences;1"]
                         .getService(Components.interfaces.nsIPrefBranch);

    // Copy old preferences to new ones
    if (nsPreferences.copyUnicharPref("irvine.edtPath", "") != "") {
      nsPreferences.setUnicharPref("extensions.irvine.edtPath",
        nsPreferences.copyUnicharPref("irvine.edtPath", ""));
    }
    if (nsPreferences.getBoolPref("irvine.flat", false)) {
      nsPreferences.setBoolPref("extensions.irvine.flat", true);
    }

    // Delete old preferences
    try {
      pref.clearUserPref("irvine.edtPath");
    } catch(e) {}
    try {
      pref.clearUserPref("irvine.flat");
    } catch(e) {}

    // Set default context menu items
    if (nsPreferences.getBoolPref("extensions.irvine.menu.download") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.download", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.immediate") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.immediate", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.add") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.add", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.saveas") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.saveas", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.withqueue") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.withqueue", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.selection") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.selection", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.withcomment") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.withcomment", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.withsetting") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.withsetting", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.all") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.all", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.allwithimg") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.allwithimg", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.import") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.import", true);
    }
    if (nsPreferences.getBoolPref("extensions.irvine.menu.thispage") == null) {
      nsPreferences.setBoolPref("extensions.irvine.menu.thispage", true);
    }

    // Set default accesskeys
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.irvine", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.irvine", "e");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.download", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.download", "a");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.immediate", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.immediate", "b");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.add", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.add", "c");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.saveas", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.saveas", "d");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.withqueue", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.withqueue", "e");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.selection", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.selection", "f");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.withcomment", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.withcomment", "g");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.withsetting", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.withsetting", "h");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.all", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.all", "i");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.allwithimg", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.allwithimg", "j");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.import", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.import", "k");
    }
    if (nsPreferences.copyUnicharPref(
          "extensions.irvine.accesskey.thispage", null) == null) {
      nsPreferences.setUnicharPref("extensions.irvine.accesskey.thispage", "l");
    }

    // Add context menu event listner
    if (menu) {
      menu.addEventListener("popupshowing", this.irvineContextListener, false);
    }
  },

  irvineContextListener: function(aEvent) {
    return(IrvineService.showItems(aEvent));
  },

  irvineInitListener: function(aEvent) {
    return(IrvineService.initItems(aEvent));
  },

  irvineRemoveListener: function(aEvent) {
    // Remove context menu event listner
    var menu = document.getElementById("contentAreaContextMenu");
    if (menu) {
      menu.removeEventListener("popupshowing", this.irvineContextListener,
                               false);
    }
  }
};

window.addEventListener("load", IrvineService.irvineInitListener, false);
window.addEventListener("unload", IrvineService.irvineRemoveListener, false);
